
# q-expansion of newform 147.8.e.b, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 147
weight = 8
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[50, [1, 0]], [52, [0, -1]]]
aps_data = [[0, -6], [27, -27], [0, -390], [0, 0], [948, -948], [-5098, 0], [-28386, 28386], [0, 8620], [0, 15288], [36510, 0], [276808, -276808], [0, -268526], [-629718, 0], [685772, 0], [0, -583296], [428058, -428058], [-1306380, 1306380], [0, -300662], [507244, -507244], [5560632, 0], [-1369082, 1369082], [0, 6913720], [-4376748, 0], [0, 8528310], [-8826814, 0], [-11986422, 11986422], [0, -7209392], [0, -11426076], [-4020950, 4020950], [-17706318, 0], [16788296, 0], [0, -16826772], [-28044906, 28044906], [-11827300, 0], [0, -20784630], [-76112, 76112], [32182474, -32182474], [0, -58343492], [-25836504, 0], [0, -63520062], [80955900, -80955900], [64503182, 0], [0, -56827392], [-116376962, 116376962], [-118816074, 0], [95010640, -95010640], [179246372, 0], [-206536888, 0], [-43395396, 43395396], [0, 36193090], [0, -92234682], [49846800, 0], [-199374002, 199374002], [-394677828, 0], [0, 142885374], [-440240712, 440240712], [-275404590, 275404590], [0, 424669768], [-516158126, 516158126], [-311042838, 0], [594308308, -594308308], [115514742, 0], [-260599804, 0], [-576794712, 576794712], [0, 460074358], [0, -62556126], [0, -684235772], [-626312974, 0], [1253396004, -1253396004], [265350230, 0], [569635518, -569635518], [0, -932540760], [852564664, -852564664], [0, -381183422], [-1483530820, 0], [0, 761929728], [-1609017990, 1609017990], [0, -1880157926], [0, -268592322], [-89947850, 89947850], [1690542660, 0], [-1133327458, 0], [-2199426912, 2199426912], [-151737838, 0], [0, -990762560], [0, 1773756228], [-277010190, 0], [0, -2947584986], [-2766873378, 0], [463552952, 0], [0, 417921804], [1509728880, -1509728880], [-929459936, 929459936], [5128027932, 0], [0, 410649340], [5020408872, 0], [0, 3249259170], [2109501558, -2109501558], [0, 5289111988], [0, -3046135142], [-4855369564, 0], [-1277621406, 1277621406], [-4712645172, 4712645172], [0, -4577997210], [-4951192172, 4951192172], [-8518472066, 8518472066], [-562247364, 0], [0, -3621104082], [7481036760, -7481036760], [-5812702678, 0], [0, -3840510056], [-1704841742, 1704841742], [-2808092454, 0], [2543654980, -2543654980], [-151146448, 0], [12362510478, -12362510478], [2867439812, 0], [4106397384, -4106397384], [0, -6910997502], [3424439940, 0], [6764370178, -6764370178], [-1749591838, 0], [0, -8300111046], [12123151668, -12123151668], [0, -8218456532], [4722309342, 0], [0, -2789746910], [0, -1519846560], [-8117607664, 0], [0, 10324065658], [13536483820, -13536483820], [-17193592248, 0], [0, -11247841832], [16306837166, 0], [0, -6140694522], [24506888450, 0], [10172166858, -10172166858], [9791348044, -9791348044], [-9757815954, 0], [2787060150, -2787060150], [-7999831348, 0], [0, 10240182138], [-27868227152, 27868227152], [23512488876, 0], [12859810810, -12859810810], [-7998323880, 0], [4208270462, 0], [-31930660986, 31930660986], [0, -21800201180], [0, -10472788512], [0, 17778707194], [-7642529838, 0], [-27637547308, 0], [0, -32308710456], [-22714240076, 22714240076], [7509250032, 0], [0, 24937401040], [0, 8662051470], [28265471066, 0], [46708171458, -46708171458], [0, 46739235324], [38242017882, 0], [-49001211184, 0], [0, -27292866732], [-3944816706, 3944816706], [-474320472, 474320472], [-12219675032, 12219675032], [36069016114, -36069016114]]
